@@ -1,12 +1,12 @@
 /*!  \file 
 
-$Header: /cvsroot/tsp/tsp/src/providers/bb_provider/bb_tsp_provider_main.c,v 1.3 2004/10/26 23:46:54 erk Exp $
+$Header $
 
 -----------------------------------------------------------------------
 
 TSP Library - core components for a generic Transport Sampling Protocol.
 
-Copyright (c) 2002 Yves DUFRENNE, Stephane GALLES, Eric NOULARD and Robert PAGNOT 
+Copyright (c) 2005 Cesare BERTONA
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -25,15 +25,16 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 -----------------------------------------------------------------------
 
 Project   : TSP
-Maintainer : tsp@astrium-space.com
-Component : Consumer
+Maintainer: tsp@astrium.eads.net
+Component : external
 
 -----------------------------------------------------------------------
 
-Purpose   : Blackboard TSP Provider
+Purpose   : posix implementation for VXWORKS
 
 -----------------------------------------------------------------------
- */
+*/
+
 #include <stdio.h>
 #include <malloc.h>
 #include <unistd.h>
@@ -44,7 +45,9 @@ Purpose   : Blackboard TSP Provider
 
 #include "bb_tsp_provider.h"
 #include "tsp_provider_init.h"
+#include "tsp_const_def.h"
 
+#ifndef VXWORKS
 
 int 
 main (int argc, char ** argv) {
@@ -57,6 +60,8 @@ main (int argc, char ** argv) {
   sigprocmask(SIG_BLOCK, &allsigs, NULL);
 
   if (argc<2) {
+    fprintf(stderr, 
+	    "TSP bb_provider v%s (%s)\n",TSP_SOURCE_VERSION,TSP_PROJECT_URL);
     fprintf(stderr,"%s : argument missing\n",argv[0]);
     fprintf(stderr,"Usage: %s <bbname>\n",argv[0]);
     exit(-1);
@@ -70,3 +75,61 @@ main (int argc, char ** argv) {
   
   return 0;
 }
+
+#else
+#include <bb_core.h>
+#include <bb_simple.h>
+#include <bb_tools.h>
+
+
+int launch_bb_vx_provider (char * bb_name) 
+{
+  int	argc = 2;
+  char	**argv;
+
+  argv = malloc(2*sizeof(char *));
+
+  argv[0] = strdup("launch_bb_vx_provider");
+  argv[1] = bb_name;
+
+  bb_simple_synchro_config(1);  /* FIXME : try to use mode PROCESS  */
+				/* 0 = BB_SIMPLE_SYNCHRO_PROCESS    */
+				/* 1 = BB_SIMPLE_SYNCHRO_THREAD     */
+
+  bb_tsp_provider_initialise(&argc,&argv,
+			     TSP_ASYNC_REQUEST_SIMPLE | TSP_ASYNC_REQUEST_NON_BLOCKING,
+			     argv[1]);
+  return 0;
+}
+
+int debug_bb(char * cmd, char * bb_name, char * sym_name, char * value)
+{
+  bbtools_request_t  the_request;
+
+
+  the_request.argv[0] = bb_name;
+  the_request.verbose = 1;
+  the_request.silent = 0;
+
+  /* initialize bbtools request */
+  bbtools_init(&the_request);
+  the_request.theBB=bbtools_checkbbname(the_request.argv[0]);
+  
+  if(strcmp("READ", cmd) == 0) {
+    the_request.argc = 2;
+    the_request.argv[1] = sym_name;
+    bbtools_read(&the_request);
+  } else if(strcmp("WRITE", cmd) == 0) {
+    the_request.argc = 3;
+    the_request.argv[1] = sym_name;
+    the_request.argv[2] = value;
+    bbtools_write(&the_request);
+  } else if(strcmp("DUMP", cmd) == 0) {
+    the_request.argc = 1;
+    bbtools_dump(&the_request);
+  }
+  
+  return 0;
+}
+
+#endif
