@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_core.c,v 1.15 2005/08/14 22:39:36 erk Exp $
+$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_core.c,v 1.17 2005/10/23 09:46:06 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -59,7 +59,7 @@ Purpose   : Blackboard Idiom implementation
 /**
  * Convert type to string for display use.
  */
-static const char* E_BB_2STRING[] = {"NoAType 0",
+static const char* E_BB_2STRING[] = {"DiscoverType",
 				     "double", 
 				     "float", 
 				     "int8_t",
@@ -75,6 +75,73 @@ static const char* E_BB_2STRING[] = {"NoAType 0",
 				     "UserType",
 				     "NotAType end"};
 
+static const size_t E_BB_TYPE_SIZE[] = {0,
+					sizeof(double), 
+					sizeof(float), 
+					sizeof(int8_t),
+					sizeof(int16_t),
+					sizeof(int32_t),
+					sizeof(int64_t),
+					sizeof(uint8_t),
+					sizeof(uint16_t),
+					sizeof(uint32_t),
+					sizeof(uint64_t),
+					sizeof(char),
+					sizeof(unsigned char),
+					0,
+					0};
+
+size_t 
+sizeof_bb_type(E_BB_TYPE_T bb_type) {
+  size_t retval = 0;
+  if ((bb_type>=1) && (bb_type<E_BB_USER)) {
+    retval = E_BB_TYPE_SIZE[bb_type];
+  }
+  return retval;
+}
+
+E_BB_TYPE_T 
+bb_type_string2bb_type(const char* bb_type_string) {
+  E_BB_TYPE_T retval = 0;
+  if (!strncasecmp("double",bb_type_string,strlen("double"))) {
+    retval = E_BB_DOUBLE;
+  } else
+  if (!strncasecmp("float",bb_type_string,strlen("float"))) {
+    retval = E_BB_FLOAT;
+  } else
+  if (!strncasecmp("int8",bb_type_string,strlen("int8"))) {
+    retval = E_BB_INT8;
+  } else
+  if (!strncasecmp("int16",bb_type_string,strlen("int16"))) {
+    retval = E_BB_INT16;
+  } else
+  if (!strncasecmp("int32",bb_type_string,strlen("int32"))) {
+    retval = E_BB_INT32;
+  } else
+  if (!strncasecmp("int64",bb_type_string,strlen("int64"))) {
+    retval = E_BB_INT64;
+  } else
+  if (!strncasecmp("uint8",bb_type_string,strlen("uint8"))) {
+    retval = E_BB_UINT8;
+  } else
+  if (!strncasecmp("uint16",bb_type_string,strlen("uint16"))) {
+    retval = E_BB_UINT16;
+  } else
+  if (!strncasecmp("uint32",bb_type_string,strlen("uint32"))) {
+    retval = E_BB_UINT32;
+  } else
+  if (!strncasecmp("uint64",bb_type_string,strlen("uint64"))) {
+    retval = E_BB_UINT64;
+  } else
+  if (!strncasecmp("char",bb_type_string,strlen("char"))) {
+    retval = E_BB_CHAR;
+  } else
+  if (!strncasecmp("uchar",bb_type_string,strlen("uchar"))) {
+    retval = E_BB_UCHAR;
+  }
+  return retval;
+}
+
 int32_t
 bb_check_version(volatile S_BB_T* bb) {
   int32_t retval = 0;
@@ -267,14 +334,18 @@ bb_value_write(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc,const char* value,
   char* data;
   int retval;
   int hexval;
+  int lenval;
   assert(bb);
   
   retval = E_OK;
 
+  lenval = strlen(value);
+
   if ((NULL != strstr(value,"0x")) | 
       (NULL != strstr(value,"0X"))
       ) {
-    hexval = 1;
+    hexval  = 1;
+    lenval -= 2;
   } else {
     hexval = 0;
   }
@@ -282,49 +353,58 @@ bb_value_write(volatile S_BB_T* bb, S_BB_DATADESC_T data_desc,const char* value,
   /* on recupere l'adresse de la donnee dans le BB */
   data = (char*)bb_data(bb) + data_desc.data_offset;
   switch (data_desc.type) {
-    case E_BB_DOUBLE: 
-      ((double *)data)[idx] = atof(value);
-      break;
-    case E_BB_FLOAT:
-      ((float *)data)[idx] = atof(value);
-      break;
-    case E_BB_INT8:
-      ((int8_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
-      break; 
-    case E_BB_INT16:
-      ((int16_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
-      break; 
-    case E_BB_INT32:
-      ((int32_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
-      break; 
-    case E_BB_INT64:
-      ((int64_t*)data)[idx] = strtoll(value,(char **)NULL,hexval ? 16 : 10);
-      break; 
-    case E_BB_UINT8:
-      ((uint8_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
-      break; 
-    case E_BB_UINT16:
-      ((uint16_t*)data)[idx] = strtoul(value,(char **)NULL,hexval ? 16 : 10);
-      break;
-    case E_BB_UINT32:
-      ((uint32_t*)data)[idx] = strtoul(value,(char **)NULL,hexval ? 16 : 10);
-      break;	
-    case E_BB_UINT64:
-      ((uint64_t*)data)[idx] = strtoull(value,(char **)NULL,hexval ? 16 : 10);
-      break;	
-    case E_BB_CHAR:
-      retval = E_NOK;
-      break;
-    case E_BB_USER:
-      retval = E_NOK;
-      break; 
-    default:
-      retval = E_NOK;
-      break;
-    }
+  case E_BB_DOUBLE: 
+    ((double *)data)[idx] = atof(value);
+    break;
+  case E_BB_FLOAT:
+    ((float *)data)[idx] = atof(value);
+    break;
+  case E_BB_INT8:
+    ((int8_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
+    break; 
+  case E_BB_INT16:
+    ((int16_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
+    break; 
+  case E_BB_INT32:
+    ((int32_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
+    break; 
+  case E_BB_INT64:
+    ((int64_t*)data)[idx] = strtoll(value,(char **)NULL,hexval ? 16 : 10);
+    break; 
+  case E_BB_UINT8:
+    ((uint8_t*)data)[idx] = strtol(value,(char **)NULL,hexval ? 16 : 10);
+    break; 
+  case E_BB_UINT16:
+    ((uint16_t*)data)[idx] = strtoul(value,(char **)NULL,hexval ? 16 : 10);
+    break;
+  case E_BB_UINT32:
+    ((uint32_t*)data)[idx] = strtoul(value,(char **)NULL,hexval ? 16 : 10);
+    break;	
+  case E_BB_UINT64:
+    ((uint64_t*)data)[idx] = strtoull(value,(char **)NULL,hexval ? 16 : 10);
+    break;	
+  case E_BB_CHAR:
+    memcpy(&((char*)data)[idx],value,sizeof(char));
+    break;
+  case E_BB_UCHAR:
+    memcpy(&((unsigned char*)data)[idx],value,sizeof(unsigned char));
+    break; 
+  case E_BB_USER:
+    
+    retval = bb_utils_convert_string2hexbuf(hexval ? value+2 : value,
+					    &((unsigned char*)data)[idx],
+					    data_desc.type_size, 
+					    hexval);
+    retval = E_NOK;
+    break; 
+  default:
+    retval = E_NOK;
+    break;
+  }
   return retval;
 }
 
+
 int32_t
 bb_data_header_print(S_BB_DATADESC_T data_desc, FILE* pf, int32_t idx) {
 
@@ -467,10 +547,8 @@ bb_create(S_BB_T** bb,
 
   /* open+create du segment SHM SysV */
   if (name_shm!=NULL) {
-    /* On dimensionne le segment shm a
-     * la taille d'une structure de BB +
-     * la taille du tableau descripteur de donnee +
-     * la taille de la zone de donnee
+    /* We size SHM segment using the size of a BB
+     * as returned by bb_size.
      */
     mmap_size = bb_size(n_data,data_size);    
     fd_shm = shmget(bb_utils_ntok(name_shm), mmap_size, IPC_CREAT|IPC_EXCL|BB_SHM_ACCESS_RIGHT);        
@@ -793,7 +871,8 @@ bb_publish(volatile S_BB_T *bb, S_BB_DATADESC_T* data_desc) {
   assert(data_desc);
   
   /* Verify that the published data is not already published
-   * (key unicity)
+   * (key unicity) and trigger automatic subscribe
+   * if key already exists.
    */
   bb_lock(bb);
   if (bb_find(bb,data_desc->name) != -1) {
@@ -803,31 +882,31 @@ bb_publish(volatile S_BB_T *bb, S_BB_DATADESC_T* data_desc) {
     retval = bb_subscribe(bb,data_desc);
     bb_lock(bb);
   } else {
-    /* calcul de la taille demandee */
+    /* compute required data size  */
     needed_size = data_desc->type_size*data_desc->dimension;    
-    /* verification espace disponible dans BB */
+    /* verify available space in BB data descriptor zone */
     if (bb->n_data >= bb->max_data_desc_size) {
       bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_publish", 
 		"No more room in BB data descriptor!! [current n_data=%d]",
 		bb->n_data);
-      /* verification taille donnee allouable */
+      /* verify available space in BB data zone */
     } else if ((bb->max_data_size-bb->data_free_offset) < needed_size) {
       bb_logMsg(BB_LOG_SEVERE,"BlackBoard::bb_publish", 
 		"No more room in BB data zone!! [left <%d> byte(s) out of <%d> required]",
 		bb->max_data_size-bb->data_free_offset,needed_size);
     } else {     
-      /* Calcule de l'adresse libre */
+      /* Compute the free address */
       retval = (char*) bb_data(bb) + bb->data_free_offset;
-      /* Mise a jour descripteur en retour */
+      /* Update returned data descriptor */
       data_desc->data_offset = bb->data_free_offset;
-      /* Mise a jour prochaine @ libre */
+      /* Update next free address */
       bb->data_free_offset  = bb->data_free_offset + needed_size;
-      /* Mise a jour descripteur de donnee */
+      /* Update data descriptor zone */
       bb_data_desc(bb)[bb->n_data] = *data_desc;
-      /* On augmente le nombre de donnees */
+      /* Increment number of published data */
       bb->n_data++;
     }
-    /* initialisation  des valeurs par defaut */
+    /* initialize publish data zone with default value */
     bb_data_initialise(bb,data_desc,NULL);
   }    
   /* no init in case of automatic subscribe */  
@@ -847,17 +926,33 @@ bb_subscribe(volatile S_BB_T *bb,
   assert(bb);
   assert(data_desc);
   
-  /* on cherche la donnee publiee par sa clef */
+  /* We seek the data using its key (name) */
   bb_lock(bb);
   idx = bb_find(bb,data_desc->name);
   if (idx==-1) {
     retval = NULL;      
   } else {
-    data_desc->type            = (bb_data_desc(bb)[idx]).type;
-    data_desc->dimension       = (bb_data_desc(bb)[idx]).dimension;
-    data_desc->type_size       = (bb_data_desc(bb)[idx]).type_size;
+    if (E_BB_DISCOVER == data_desc->type) {
+      data_desc->type            = (bb_data_desc(bb)[idx]).type;
+    }
+
+    if (0 == data_desc->dimension) {
+      data_desc->dimension       = (bb_data_desc(bb)[idx]).dimension;
+    }
+
+    if (0 == data_desc->type_size) {
+      data_desc->type_size       = (bb_data_desc(bb)[idx]).type_size;
+    }
+
     data_desc->data_offset     = (bb_data_desc(bb)[idx]).data_offset;
-    retval = (char*) bb_data(bb) + data_desc->data_offset;
+    /* return NULL pointer if symbol signature does not match */
+    if ((data_desc->type      !=  (bb_data_desc(bb)[idx]).type)     ||
+	(data_desc->dimension != (bb_data_desc(bb)[idx]).dimension) ||
+	(data_desc->type_size != (bb_data_desc(bb)[idx]).type_size)) {
+      retval = NULL;
+    } else {
+      retval = (char*) bb_data(bb) + data_desc->data_offset;
+    }
   }
   bb_unlock(bb);
 
@@ -882,7 +977,7 @@ bb_dump(volatile S_BB_T *bb,FILE* p_filedesc) {
 	  bb->n_data,
 	  bb->max_data_desc_size);
   fprintf(p_filedesc,"  free data size       = %ld / %ld\n",
-	  bb->max_data_size - bb->data_free_offset + 1,
+	  bb->max_data_size - bb->data_free_offset,
 	  bb->max_data_size);  
   fprintf(p_filedesc,"  @data_desc           = 0x%x\n",
 	  (unsigned int) (bb_data_desc(bb)));
