@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_core.h,v 1.12 2005/08/14 22:39:36 erk Exp $
+$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_core.h,v 1.15 2005/10/23 09:46:06 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -88,7 +88,8 @@ union semun {
  * BlackBoard publishable data type.
  * @ingroup BlackBoard
  */
-typedef enum {E_BB_DOUBLE=1, 
+typedef enum {E_BB_DISCOVER=0,
+              E_BB_DOUBLE=1, 
 	      E_BB_FLOAT,
 	      E_BB_INT8, 
 	      E_BB_INT16, 
@@ -208,6 +209,12 @@ typedef struct S_BB {
 
 BEGIN_C_DECLS
 
+size_t 
+sizeof_bb_type(E_BB_TYPE_T bb_type);
+
+E_BB_TYPE_T 
+bb_type_string2bb_type(const char* bb_type_string);
+
 /**
  * Check if the accessed blackboard is of the same
  * version as the one used by this code.
@@ -221,8 +228,9 @@ bb_check_version(volatile S_BB_T* bb);
 
 /**
  * Return the size of a blackboard with 
- * n_data publishable elements
- * with a data zone of size data_size byte.
+ * n_data publishable elements with a data zone of size data_size byte.
+ * This is the exact size of the SHM segment required
+ * to create a BB.
  * @param n_data IN the number of publishable data in blackboard
  * @param data_size IN the size (in byte) of the blackboard data zone.
  * @return the size (in byte) of this kind of blackboard.
@@ -241,6 +249,30 @@ bb_size(const int32_t n_data, const int32_t data_size);
 int32_t 
 bb_find(volatile S_BB_T* bb, const char* var_name);
 
+/**
+ * Initialize the fast find string algorithm.
+ * The function will allocate a big string
+ * which is the concatenation of the name
+ * of the published BB element.
+ * @return fastfind string to be used for fast finding.
+ */
+/* char* */
+/* bb_fastfind_build(volatile S_BB_T* bb); */
+
+/**
+ * Search a variable within a BlackBoard using fastfind algorithm.
+ * @param bb IN BlackBoard pointer
+ * @param fastfind_string IN the fastfind string build with @ref bb_fastfind_build
+ * @param var_name IN the name of the searched variable
+ * @return index of the variable in the BB descriptor array
+ * @ingroup BlackBoard
+ */
+/* int32_t */
+/* bb_fastfind(volatile S_BB_T* bb, const char* fastfind_string, const char* var_name); */
+
+/* int32_t */
+/* bb_fastfind_destroy(volatile S_BB_T* bb, char* fastfindstring); */
+
 /**
  * Return a pointer to the beginning of the data 
  * descriptor array of the specified BB.
