@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_utils.h,v 1.4 2005/02/23 17:28:11 dufy Exp $
+$Header: /cvsroot/tsp/tsp/src/util/libbb/bb_utils.h,v 1.6 2005/10/23 12:28:56 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -62,6 +62,7 @@ typedef enum BB_LOG_LEVEL {
   BB_LOG_FINER
 } BB_LOG_LEVEL_T;
 
+BEGIN_C_DECLS
 
 /**
  * Build the string name used for creating a shm segment
@@ -145,6 +146,21 @@ key_t
 bb_utils_ntok(const char* name);
 
 
+/**
+ * Write the value represented as string in a buffer
+ * of unsigned char. The value is understood as
+ * decimal of hexadecimal value representation
+ * @param string IN, the value to be written to buf
+ * @param buf OUT, the buffer to be written to
+ * @param buflen IN, the buffer length
+ * @param hexval IN, 0 if string is to be interpreted as decimal value
+ *                   1 if string is to be interpreted as hexadecimal value
+ * @return 0 if success !=0 otherwise
+ * @ingroup BBUtils
+ */
+int32_t
+bb_utils_convert_string2hexbuf(const char* string, unsigned char* buf, size_t buflen, int hexval);
+
 /**
  * Log message for BB Error.
  * This function may be replaced by a project specific function
@@ -159,4 +175,6 @@ bb_utils_ntok(const char* name);
 int32_t 
 bb_logMsg(const BB_LOG_LEVEL_T level, const char* modname, char* fmt, ...);
 
+END_C_DECLS
+
 #endif /* _BB_UTILS_H_ */
